@@ -14,13 +14,15 @@
 #include <iostream>
 #include <fstream>
 #include <getopt.h>
+#include <seqan/sequence.h>
+#include <seqan/file.h>
+
 #include "bwt_map.h"
 #include "inserts.h"
 #include "closures.h"
 #include "reads.h"
 #include "tokenize.h"
-#include <seqan/sequence.h>
-#include <seqan/file.h>
+#include "fusions.h"
 
 using namespace seqan;
 using namespace std;
@@ -30,13 +32,9 @@ bool possible_cotranscript(const BowtieHit& h1, const BowtieHit& h2, bool check_
 {
   if (h1.insert_id() != h2.insert_id()) 
     return false;
-  int min_mate_inner_dist = inner_dist_mean - inner_dist_std_dev;
-  if (max_mate_inner_dist == -1)
-    {
-      max_mate_inner_dist = inner_dist_mean + inner_dist_std_dev;
-    }
-  
-  InsertAlignmentGrade grade(h1,h2, min_mate_inner_dist, max_mate_inner_dist);
+
+  JunctionSet junctions;
+  InsertAlignmentGrade grade(h1, h2, junctions);
   return (!grade.too_far && !grade.too_close && grade.opposite_strands);
 }
 
@@ -84,6 +82,77 @@ void check_mates(const HitList& hits1_in_ref,
     }	
 }
 
+void find_fusion_closure(HitsForRead& left_hits,
+			 HitsForRead& right_hits,
+			 std::set<Fusion>& fusions)
+{
+  if (left_hits.hits.size() > 3 || right_hits.hits.size() > 3)
+    return;
+
+  for (size_t left_index = 0; left_index < left_hits.hits.size(); ++left_index)
+    {
+      for (size_t right_index = 0; right_index < right_hits.hits.size(); ++right_index)
+	{
+	  BowtieHit* leftHit = &left_hits.hits[left_index];
+	  BowtieHit* rightHit = &right_hits.hits[right_index];
+
+	  uint32_t dir = FUSION_FF;
+	  if (leftHit->antisense_align() != rightHit->antisense_align())
+	    dir = FUSION_FF;
+	  else if (!leftHit->antisense_align())
+	    dir = FUSION_FR;
+	  else
+	    dir = FUSION_RF;
+
+	  if (dir == FUSION_FF && leftHit->antisense_align())
+	    {
+	      BowtieHit * tmp = leftHit;
+	      leftHit = rightHit;
+	      rightHit = tmp;
+	    }
+	  
+	  uint32_t left, right;
+	  if (dir == FUSION_FF || dir == FUSION_FR)
+	    left = leftHit->right() - 4;
+	  else
+	    left = leftHit->left() + 4;
+	  
+	  if (dir == FUSION_FF || dir == FUSION_RF)
+	    right = rightHit->left() + 4;
+	  else
+	    right = rightHit->right() - 4;
+
+	  if (leftHit->ref_id() == rightHit->ref_id() && dir == FUSION_FF)
+	    {
+	      int dist = 0;
+	      dist = rightHit->left() - leftHit->right();
+	      
+	      if (dist >= 0 && dist <= (int)fusion_min_dist)
+		continue;
+	    }
+	  
+	  uint32_t ref_id1 = leftHit->ref_id();
+	  uint32_t ref_id2 = rightHit->ref_id();
+	  
+	  if (dir == FUSION_FR || dir == FUSION_RF)
+	    {
+	      if ((ref_id2 < ref_id1) || (ref_id1 == ref_id2 && left > right))
+		{
+		  uint32_t temp = ref_id1;
+		  ref_id1 = ref_id2;
+		  ref_id2 = temp;
+		  
+		  temp = left;
+		  left = right;
+		  right = temp;
+		}
+	    }
+
+	  fusions.insert(Fusion(ref_id1, ref_id2, left, right, dir));
+	}
+    }
+}
+
 bool prefer_shorter_pairs = true;
 
 typedef pair<InsertAlignmentGrade, vector<InsertAlignment> > BestPairingHits; 
@@ -106,13 +175,9 @@ void visit_best_pairing(HitsForRead& left_hit_group,
 	    continue;
 	  
 	  uint32_t refid = h1.ref_id();
-	  
-	  int min_mate_inner_dist = inner_dist_mean - inner_dist_std_dev;
-	  if (max_mate_inner_dist == -1)
-	    {
-	      max_mate_inner_dist = inner_dist_mean + inner_dist_std_dev;
-	    }
-	  InsertAlignmentGrade s(h1, h2, min_mate_inner_dist, max_mate_inner_dist);
+
+	  JunctionSet junctions;
+	  InsertAlignmentGrade s(h1, h2, junctions);
 	  
 	  //pair<InsertAlignmentGrade, vector<InsertAlignment> >& insert_best
 	  //					= best_status_for_inserts[curr_left_obs_order];
@@ -437,17 +502,20 @@ public:
   }
 };
 
-void closure_driver(vector<FILE*> map1, 
-		    vector<FILE*> map2, 
+void closure_driver(vector<FZPipe>& map1, 
+		    vector<FZPipe>& map2, 
 		    ifstream& ref_stream, 
-		    FILE* juncs_file)
+		    FILE* juncs_file,
+		    FILE* fusions_out)
 {
   typedef RefSequenceTable::Sequence Reference;
   
   ReadTable it;
   RefSequenceTable rt(true);
-  
-  BowtieHitFactory hit_factory(it,rt);
+
+  BowtieHitFactory hit_factory(it, rt);
+
+  std::set<Fusion> fusions;
   
   fprintf (stderr, "Finding near-covered motifs...");
   CoverageMapVisitor cov_map_visitor(ref_stream, rt);
@@ -456,8 +524,8 @@ void closure_driver(vector<FILE*> map1,
   assert(map1.size() == map2.size());
   for (size_t num = 0; num < map1.size(); ++num)
     {
-      HitStream left_hs(map1[num], &hit_factory, false, true, false);
-      HitStream right_hs(map2[num], &hit_factory, false, true, false);
+      HitStream left_hs(map1[num].file, &hit_factory, false, true, false);
+      HitStream right_hs(map2[num].file, &hit_factory, false, true, false);
       
       HitsForRead curr_left_hit_group;
       HitsForRead curr_right_hit_group;
@@ -468,11 +536,11 @@ void closure_driver(vector<FILE*> map1,
       uint32_t curr_right_obs_order = it.observation_order(curr_left_hit_group.insert_id);
       uint32_t curr_left_obs_order = it.observation_order(curr_right_hit_group.insert_id);
       
-      while(curr_left_obs_order != 0xFFFFFFFF && 
-	    curr_right_obs_order != 0xFFFFFFFF)
+      while(curr_left_obs_order != VMAXINT32 &&
+	    curr_right_obs_order != VMAXINT32)
 	{
 	  while (curr_left_obs_order < curr_right_obs_order&&
-		 curr_left_obs_order != 0xFFFFFFFF && curr_right_obs_order != 0xFFFFFFFF)
+		 curr_left_obs_order != VMAXINT32 && curr_right_obs_order != VMAXINT32)
 	    {
 	      // Get hit group
 	      
@@ -481,7 +549,7 @@ void closure_driver(vector<FILE*> map1,
 	    }
 	  
 	  while (curr_left_obs_order > curr_right_obs_order &&
-		 curr_left_obs_order != 0xFFFFFFFF && curr_right_obs_order != 0xFFFFFFFF)
+		 curr_left_obs_order != VMAXINT32 && curr_right_obs_order != VMAXINT32)
 	    {
 	      // Get hit group
 	      
@@ -490,10 +558,14 @@ void closure_driver(vector<FILE*> map1,
 	    }
 	  
 	  while (curr_left_obs_order == curr_right_obs_order &&
-		 curr_left_obs_order != 0xFFFFFFFF && curr_right_obs_order != 0xFFFFFFFF)
-	    {			
-	      if (coverage_attempts++ % 1000 == 0)
-		fprintf (stderr, "Adding covered motifs from pair %d\n", coverage_attempts); 
+		 curr_left_obs_order != VMAXINT32 && curr_right_obs_order != VMAXINT32)
+	    {
+	      if (num == 0)
+		find_fusion_closure(curr_left_hit_group, curr_right_hit_group, fusions);
+	      
+	      if (coverage_attempts++ % 10000 == 0)
+		fprintf (stderr, "Adding covered motifs from pair %d\n", coverage_attempts);
+
 	      visit_best_pairing(curr_left_hit_group, curr_right_hit_group, cov_map_visitor);
 	      
 	      left_hs.next_read_hits(curr_left_hit_group);
@@ -517,11 +589,11 @@ void closure_driver(vector<FILE*> map1,
   
   for (size_t num = 0; num < map1.size(); ++num)
     {
-      rewind(map1[num]);
-      rewind(map2[num]);
+      map1[num].rewind();
+      map2[num].rewind();
       
-      HitStream left_hs = HitStream(map1[num], &hit_factory, false, true, false);
-      HitStream right_hs = HitStream(map2[num], &hit_factory, false, true, false);
+      HitStream left_hs = HitStream(map1[num].file, &hit_factory, false, true, false);
+      HitStream right_hs = HitStream(map2[num].file, &hit_factory, false, true, false);
       
       HitsForRead curr_left_hit_group;
       HitsForRead curr_right_hit_group;
@@ -532,11 +604,11 @@ void closure_driver(vector<FILE*> map1,
       uint32_t curr_right_obs_order = it.observation_order(curr_left_hit_group.insert_id);
       uint32_t curr_left_obs_order = it.observation_order(curr_right_hit_group.insert_id);
       
-      while(curr_left_obs_order != 0xFFFFFFFF && 
-	    curr_right_obs_order != 0xFFFFFFFF)
+      while(curr_left_obs_order != VMAXINT32 &&
+	    curr_right_obs_order != VMAXINT32)
 	{
 	  while (curr_left_obs_order < curr_right_obs_order &&
-		 curr_left_obs_order != 0xFFFFFFFF && curr_right_obs_order != 0xFFFFFFFF)
+		 curr_left_obs_order != VMAXINT32 && curr_right_obs_order != VMAXINT32)
 	    {
 	      // Get hit group
 	      
@@ -545,7 +617,7 @@ void closure_driver(vector<FILE*> map1,
 	    }
 	  
 	  while (curr_left_obs_order > curr_right_obs_order &&
-		 curr_left_obs_order != 0xFFFFFFFF && curr_right_obs_order != 0xFFFFFFFF)
+		 curr_left_obs_order != VMAXINT32 && curr_right_obs_order != VMAXINT32)
 	    {
 	      // Get hit group
 	      
@@ -554,10 +626,11 @@ void closure_driver(vector<FILE*> map1,
 	    }
 	  
 	  while (curr_left_obs_order == curr_right_obs_order &&
-		 curr_left_obs_order != 0xFFFFFFFF && curr_right_obs_order != 0xFFFFFFFF)
+		 curr_left_obs_order != VMAXINT32 && curr_right_obs_order != VMAXINT32)
 	    {	
-	      if (closure_attempts++ % 1000 == 0)
-		fprintf (stderr, "Trying to close pair %d\n", closure_attempts); 
+	      if (closure_attempts++ % 10000 == 0)
+		fprintf (stderr, "Trying to close pair %d\n", closure_attempts);
+
 	      visit_best_pairing(curr_left_hit_group, curr_right_hit_group, junc_map_visitor);
 	      left_hs.next_read_hits(curr_left_hit_group);
 	      curr_left_obs_order = it.observation_order(curr_left_hit_group.insert_id);
@@ -567,6 +640,12 @@ void closure_driver(vector<FILE*> map1,
 	    }
 	}
     }
+
+  for (size_t num = 0; num < map1.size(); ++num)
+    {
+      map1[num].close();
+      map2[num].close();
+    }
   
   fprintf(stderr, "%lu Forward strand splices\n", fwd_splices.size());
   fprintf(stderr, "%lu Reverse strand splices\n", rev_splices.size());
@@ -604,6 +683,36 @@ void closure_driver(vector<FILE*> map1,
   fprintf(stderr, "Successfully closed %d pairs\n", closed);
   
   fprintf(stderr, "Found %d total possible splices\n", num_potential_splices);
+
+  // daehwan
+#if 0
+  fprintf (stderr, "Reporting potential fusions...\n");
+  if(fusions_out){
+    for(std::set<Fusion>::iterator itr = fusions.begin(); itr != fusions.end(); ++itr){
+      const char* ref_name1 = rt.get_name(itr->refid1);
+      const char* ref_name2 = rt.get_name(itr->refid2);
+      
+      const char* dir = "";
+      if (itr->dir == FUSION_FR)
+	dir = "fr";
+      else if(itr->dir == FUSION_RF)
+	dir = "rf";
+      else
+	dir = "ff";
+      
+      fprintf(fusions_out,
+	      "%s\t%d\t%s\t%d\t%s\n",
+	      ref_name1,
+	      itr->left,
+	      ref_name2,
+	      itr->right,
+	      dir);
+    }
+    fclose(fusions_out);
+  }else{
+    fprintf(stderr, "Failed to open fusions file for writing\n");
+  }
+#endif
 }
 
 void print_usage()
@@ -633,9 +742,17 @@ int main(int argc, char** argv)
       print_usage();
       return 1;
     }
+
+  string fusions_file_name = argv[optind++];
   
+  if(optind >= argc)
+    {
+      print_usage();
+      return 1;
+    }
+
   string ref_fasta = argv[optind++];
-  
+
   if(optind >= argc)
     {
       print_usage();
@@ -644,12 +761,14 @@ int main(int argc, char** argv)
 
   string left_file_list = argv[optind++];
   vector<string> left_file_names;
-  vector<FILE*> left_files;
+  vector<FZPipe> left_files;
   tokenize(left_file_list, ",", left_file_names);
+
+  string unzcmd = getUnpackCmd(left_file_names[0], false);
   for (size_t i = 0; i < left_file_names.size(); ++i)
     {
-      FILE* seg_file = fopen(left_file_names[i].c_str(), "r");
-      if (seg_file == NULL)
+      FZPipe seg_file(left_file_names[i], unzcmd);
+      if (seg_file.file == NULL)
         {
 	  fprintf(stderr, "Error: cannot open file %s for reading\n",
 		  left_file_names[i].c_str());
@@ -666,12 +785,12 @@ int main(int argc, char** argv)
   
   string right_file_list = argv[optind++];
   vector<string> right_file_names;
-  vector<FILE*> right_files;
+  vector<FZPipe> right_files;
   tokenize(right_file_list, ",", right_file_names);
   for (size_t i = 0; i < right_file_names.size(); ++i)
     {
-      FILE* seg_file = fopen(right_file_names[i].c_str(), "r");
-      if (seg_file == NULL)
+      FZPipe seg_file(right_file_names[i], unzcmd);
+      if (seg_file.file == NULL)
 	{
 	  fprintf(stderr, "Error: cannot open %s for reading\n",
 		  right_file_names[i].c_str());
@@ -689,11 +808,20 @@ int main(int argc, char** argv)
 	      junctions_file_name.c_str());
       exit(1);
     }
-  
+
+  FILE* fusion_db = fopen(fusions_file_name.c_str(), "w");
+  if (splice_db == NULL)
+    {
+      fprintf(stderr, "Error: cannot open fusions file %s for writing\n",
+	      fusions_file_name.c_str());
+      exit(1);
+    }
+
   closure_driver(left_files,
 		 right_files,
 		 ref_stream,
-		 splice_db);
+		 splice_db,
+		 fusion_db);
   
   return 0;
 }
